'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "gamelib"
package.
'''


import pyglet
from pyglet.window import *
from pyglet.gl import *
from pyglet.media import *
import scenes.home.scene
import scenes.manager as scene_manager

#Constants
WINDOWWIDTH = 800
WINDOWHEIGHT = 600
GAME_TITLE = "Most Wanted Hipsters"
VERSION = 0.1
DATAPATH = "data/"

#window and graphics
window = pyglet.window.Window(width = WINDOWWIDTH, height = WINDOWHEIGHT)
window.set_caption(GAME_TITLE + " v." + str(VERSION))
window.set_mouse_visible(True)

glEnable(GL_BLEND)
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

pyglet.clock.set_fps_limit(60);

scene_manager.goto_homescreen()

#Debug UI
fps_display = pyglet.clock.ClockDisplay()
#window.push_handlers(pyglet.window.event.WindowEventLogger())

@window.event
def on_mouse_press(x, y, button, modifiers):
    global scene_manager
    scene_manager.scene_list[-1].on_mouse_press(x, y, button, modifiers)
    
@window.event
def on_mouse_drag(x, y, dx, dy, buttons, modifiers):
    global scene_manager
    scene_manager.scene_list[-1].on_mouse_drag(x, y, dx, dy, buttons, modifiers)
    
@window.event
def on_mouse_motion(x, y, dx, dy):
    global scene_manager
    scene_manager.scene_list[-1].on_mouse_motion(x, y, dx, dy)
    
@window.event
def on_mouse_release(x, y, button, modifiers):
    global scene_manager
    scene_manager.scene_list[-1].on_mouse_release(x, y, button, modifiers)

@window.event
def on_key_press(symbol, modifiers):
    global scene_manager
    scene_manager.scene_list[-1].on_key_press(symbol, modifiers)
    
@window.event
def on_key_release(symbol, modifiers):
    global scene_manager
    scene_manager.scene_list[-1].on_key_release(symbol, modifiers)

#===============================================================================
# @window.event
# def on_text(text):
#    global scene_manager
#    scene_manager.scene_list[-1].on_text(text)
# 
# @window.event
# def on_text_motion(motion):
#    global scene_manager
#    scene_manager.scene_list[-1].on_text_motion(motion)
#===============================================================================

@window.event
def on_draw():
    global scene_manager
    window.clear()
    try:
        scene_manager.scene_list[-1].on_draw()
    except:
        pass
    fps_display.draw()
        
def update(dt):
    global scene_manager
    scene_manager.update()
   
def main():
    pyglet.clock.schedule_interval(update, 1./60.)
    pyglet.app.run()
    
