import shared.scene
from scenes.game.city import City
from scenes.game.objects.hipster import Hipsters
from scenes.game.objects.lead_hipster import LeadHipster
from scenes.game.objects.cop import Cops
import pyglet
import random
     
arrested_hipsters = 0
     
NORTH = 270
WEST = 180
SOUTH = 90
EAST = 0
     
class Game(shared.scene.Scene):
    #CONSTANTS    
    mouse_target = None
    hipsters = None
    cops = None
    norms = []
    city = None
    score = 0
    target_score = None
    
    running = 0
    
    def __init__(self, level):
        shared.scene.Scene.__init__(self)
        
        self.city = City()
        self.hipsters = Hipsters()
        self.cops = Cops()
        
        cop_start = random.choice(self.city.streets)
        for i in range(0, level.num_cops):
            self.cops.add_cop(cop_start.x, cop_start.y)
        
        for i in range(0, level.num_hipsters):
            start = random.choice([a for a in self.city.streets if a != cop_start])
            self.hipsters.add_hipster(start.x, start.y)
            
        
        start = random.choice([a for a in self.city.streets if a != cop_start])
        self.lead_hipster = LeadHipster(start.x, start.y, speed = 2)
        
        self.score = 0
        self.target_score = level.target_score
        
        self.score_text = pyglet.text.Label(str(self.score) + " miles",
                      font_name='Segoe Script',
                      font_size=24,
                      x=720, y=25,
                      height = 72,
                      width = 200,
                      anchor_x='center',
                      anchor_y='center')
        
        
        self.instruction = pyglet.text.Label('Place your guides.\nPress \'Space\' when ready.',
                      font_name='Segoe Script',
                      multiline = True,
                      font_size=24,
                      x=400, y=300,
                      height = 300,
                      width = 600,
                      anchor_x='center',
                      anchor_y='center')
        
        media = pyglet.resource.media('data/Throu-calpomat-6942_hifi.mp3', streaming=False)
        self.player = pyglet.media.Player()
        self.player.queue(media)
        self.player.eos_action = pyglet.media.Player.EOS_LOOP
        self.player.play()
            
    def on_key_press(self, key, modifiers):
        if key == pyglet.window.key.F2:
            import scenes.manager as Manager
            self.player.pause()
            Manager.goto_next_level()
        if self.running != 1 and key == pyglet.window.key.SPACE:
            self.running += 1
        if self.running == 1:
            if key == pyglet.window.key.UP:
                self.lead_hipster.target_direction = NORTH
            if key == pyglet.window.key.DOWN:
                self.lead_hipster.target_direction = SOUTH
            if key == pyglet.window.key.LEFT:
                self.lead_hipster.target_direction = WEST
            if key == pyglet.window.key.RIGHT:
                self.lead_hipster.target_direction = EAST
            
            
    def on_mouse_press(self, x, y, button, modifiers):
        if self.running == 0:
            if button is pyglet.window.mouse.LEFT:
                self.mouse_target = self.city.get_intersection(x, y, range=20)

    def on_mouse_drag(self, x, y, dx, dy, button, modifiers):
        if self.running == 0:
            if self.mouse_target:
                delt_x = self.mouse_target.x - x
                delt_y = self.mouse_target.y - y
                
                if abs(delt_x) > abs(delt_y):
                    if delt_x < 0:
                        self.mouse_target.set_state(0)
                    else:
                        self.mouse_target.set_state(180)
                else:
                    if delt_y < 0:
                        self.mouse_target.set_state(270)
                    else:
                        self.mouse_target.set_state(90)
        
    def on_mouse_release(self, x, y, button, modifiers):
        if self.running == 0:
            if button is pyglet.window.mouse.LEFT:
                self.mouse_target = None

    
    def update(self):
        if len(self.hipsters.hipsters) == 0 and self.running == 1:
            self.running = 2
            self.instruction.text = "All your hipsters have been caught! \n Press 'Space'"
            
        if  self.running >= 3:    
            import scenes.manager as Manager
            Manager.goto_homescreen()
              
        if self.running == 1: 
            self.city.update()
                        
            intersection_density = {}
            targeted_hipsters = []
            #set cop to chase closest Hipster
            for hipster in self.hipsters.get_hipsters():
                try:
                    intersection_density[self.city.get_intersection(hipster.x, hipster.y, 60)] += 1
                except:
                    intersection_density[self.city.get_intersection(hipster.x, hipster.y, 60)] = 1
                    
                for cop in self.cops.get_cops():
                    cop.stop_chase()
                    cop.target_speed = self.lead_hipster.speed
                    if cop.in_sight(hipster.x, hipster.y):
                        if hipster not in targeted_hipsters:
                            cop.start_chase(hipster)
                            targeted_hipsters.append(hipster)
                    if cop.in_range(hipster.x, hipster.y, hipster.speed * 2):
                        global arrested_hipsters
                        arrested_hipsters += 1
                        hipster.following = False
                        try:
                            del self.lead_hipster.followers[hipster]
                        except:
                            pass
                        self.hipsters.kill_hipster(hipster) 
                        cop.stop_chase() 
                        
            #set cops without chase target to go to location of highest hipster mass
            best_intersection = (None, 0)
            for intersection in intersection_density:
                if intersection_density[intersection] > best_intersection[1]:
                    best_intersection = (intersection, intersection_density[intersection])
            
            if best_intersection[0] is not None:
                for cop in self.cops.get_cops():
                    if cop.chase is False:
                        cop.start_chase(best_intersection[0])
            else:
                for cop in self.cops.get_cops():
                    if cop.chase is False:
                        cop.start_chase(random.choice(self.city.streets))
                        
            location = []
            for cop in self.cops.cops:
                cop_loc = (cop.x, cop.y)
                if cop_loc in location:
                    cop.stop_chase()
                else:
                    location.append(cop_loc)
                    
            #setup hipsters massing
            for hipster in self.hipsters.get_hipsters():
                if self.lead_hipster.intersects(hipster.x, hipster.y, hipster.speed):
                    self.lead_hipster.add_follower(hipster)
                    hipster.following = True
                    
                        
            #update lead_hipster
            intersection = self.city.get_intersection(self.lead_hipster.x, self.lead_hipster.y, self.lead_hipster.speed)
            if intersection:
                self.lead_hipster.x = intersection.x
                self.lead_hipster.y = intersection.y
                self.lead_hipster.change_direction(intersection.paths)
            self.lead_hipster.update()
            self.score += self.lead_hipster.speed

            #update the hipsters
            for hipster in self.hipsters.get_hipsters():
                if hipster.following == False:
                    intersection = self.city.get_intersection(hipster.x, hipster.y, hipster.speed)
                    if intersection:
                        hipster.x = intersection.x
                        hipster.y = intersection.y
                        hipster.change_direction(random.choice(intersection.get_paths()))
                    hipster.update()
                self.score += hipster.speed
                
            #update the cops
            for cop in self.cops.get_cops():
                intersection = self.city.get_intersection(cop.x, cop.y, cop.speed)
                if intersection:
                    cop.x = intersection.x
                    cop.y = intersection.y
                    cop.change_direction(random.choice(intersection.paths))
                cop.update()
                
            #update UI    
            self.score_text.text = str(self.score / 120 / 4) + " miles"
            
            
        if self.running == 0:
            pass
            
        #check for endgame
        if self.score / 120 / 4 >= self.target_score:
            import scenes.manager as Manager
            self.player.pause()
            Manager.goto_next_level()
        return False
    
    def on_draw(self):
        self.city.on_draw()
        self.hipsters.on_draw()
        self.lead_hipster.on_draw()
        self.cops.on_draw()
        self.score_text.draw()
        
        if self.running == 0 or self.running == 2:
            self.instruction.draw()
        
        